@@ -1,4 +1,4 @@
-/*      $Id: xfwm4_shortcuteditor.c 22213 2006-06-28 21:18:22Z olivier $
+/*      $Id: xfwm4_shortcuteditor.c 23813 2006-11-10 20:47:06Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -134,6 +134,7 @@ cb_popup_add_menu (GtkWidget * widget, gpointer data)
     dialog = gtk_dialog_new_with_buttons (_("Add keybinding theme"), GTK_WINDOW (itf->xfwm4_dialog),
         GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
+    gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
     header_image = gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_LARGE_TOOLBAR);
     header = xfce_create_header_with_image (header_image, _("Add keybinding theme"));
@@ -142,6 +143,7 @@ cb_popup_add_menu (GtkWidget * widget, gpointer data)
     hbox = gtk_hbox_new (FALSE, BORDER);
     label = gtk_label_new (_("Enter a name for the theme:"));
     entry = gtk_entry_new ();
+    gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
     gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
     gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
 
@@ -312,7 +314,8 @@ cb_popup_menu (GtkTreeView * treeview, GdkEventButton * event, gpointer data)
 void
 loadtheme_in_treeview (ThemeInfo * ti, gpointer data)
 {
-    const gchar *shortcut_options_list[] = { "close_window_key",
+    const gchar *shortcut_options_list[] = { 
+        "close_window_key",
         "maximize_window_key",
         "maximize_vert_key",
         "maximize_horiz_key",
@@ -347,10 +350,13 @@ loadtheme_in_treeview (ThemeInfo * ti, gpointer data)
         "move_window_left_workspace_key",
         "move_window_right_workspace_key",
         "show_desktop_key",
+        "cancel_key",
+        "popup_menu_key",
         NULL
     };
 
-    const gchar *shortcut_name_list[] = { N_("Close window"),
+    const gchar *shortcut_name_list[] = { 
+        N_("Close window"),
         N_("Maximize window"),
         N_("Maximize window vertically"),
         N_("Maximize window horizontally"),
@@ -385,6 +391,8 @@ loadtheme_in_treeview (ThemeInfo * ti, gpointer data)
         N_("Move window to left workspace"),
         N_("Move window to right workspace"),
         N_("Show desktop"),
+        N_("Cancel window action"),
+        N_("Window operations menu"),
         NULL
     };
 
@@ -682,7 +690,6 @@ cb_compose_shortcut (GtkWidget * widget, GdkEventKey * event, gpointer data)
     guint keyval;
     guint modifiers;
     gchar *accelerator;
-    gint i;
     gchar **shortcuts;
     gchar **current_shortcut;
 
@@ -733,14 +740,7 @@ cb_compose_shortcut (GtkWidget * widget, GdkEventKey * event, gpointer data)
     modifiers = event->state & (~consumed_modifiers | GDK_MODIFIER_MASK);
     modifiers = modifiers & gtk_accelerator_get_default_mod_mask ();
     accelerator = gtk_accelerator_name (keyval, modifiers);
-
-    for (i = 0; i < strlen (accelerator); i++)
-    {
-        if (accelerator[i] == '>')
-            accelerator[i] = '<';
-    }
-
-    shortcuts = g_strsplit (accelerator, "<", 0);
+    shortcuts = g_strsplit_set (accelerator, "<>", 0);
 
     current_shortcut = shortcuts;
     while (*current_shortcut)
@@ -845,8 +845,11 @@ cb_activate_treeview3 (GtkWidget * treeview, GtkTreePath * path, GtkTreeViewColu
 
     /* Create dialog */
     dialog =
-        gtk_dialog_new_with_buttons (_("Compose shortcut"), NULL, GTK_DIALOG_MODAL,
-        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
+        gtk_dialog_new_with_buttons (_("Compose shortcut"), NULL, GTK_DIALOG_MODAL, NULL);
+
+    button = xfce_create_mixed_button (GTK_STOCK_CANCEL, _("Cancel"));
+    gtk_widget_show (button);
+    gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_CANCEL);
 
     button = xfce_create_mixed_button (GTK_STOCK_CLEAR, _("No shortcut"));
     gtk_widget_show (button);
@@ -878,7 +881,7 @@ cb_activate_treeview3 (GtkWidget * treeview, GtkTreePath * path, GtkTreeViewColu
         GTK_BUTTONBOX_SPREAD);
 
     /* Connect signals */
-    g_signal_connect (G_OBJECT (dialog), "key-press-event", G_CALLBACK (cb_compose_shortcut), itf);
+    g_signal_connect (G_OBJECT (dialog), "key-release-event", G_CALLBACK (cb_compose_shortcut), itf);
 
     /* Take control on the keyboard */
     if (gdk_keyboard_grab (gtk_widget_get_root_window (dialog), TRUE,
