@@ -1,4 +1,4 @@
-/*      $Id: xfwm4_shortcuteditor.h,v 1.8 2004/10/12 06:50:27 olivier Exp $
+/*      $Id: xfwm4_shortcuteditor.h 20266 2006-03-06 22:05:37Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -28,6 +28,4 @@ void savetreeview_in_theme (gchar *, gpointer);
 
 void cb_activate_treeview3 (GtkWidget *, GtkTreePath *,
                             GtkTreeViewColumn *, gpointer);
-void cb_activate_treeview4 (GtkWidget *, GtkTreePath *,
-                                   GtkTreeViewColumn *, gpointer);
 #endif
