@@ -1,4 +1,4 @@
-/*      $Id: events.c 25372 2007-04-02 19:48:21Z olivier $
+/*      $Id: events.c 25649 2007-04-27 20:01:12Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -359,7 +359,7 @@ getKeyPressed (ScreenInfo *screen_info, XKeyEvent * ev)
     int key, state;
 
     state = ev->state & MODIFIER_MASK;
-    for (key = 0; key < KEY_LAST; key++)
+    for (key = 0; key < KEY_COUNT; key++)
     {
         if ((screen_info->params->keys[key].keycode == ev->keycode)
             && (screen_info->params->keys[key].modifier == state))
