@@ -1,22 +1,22 @@
-/*      $Id: events.h 21549 2006-05-06 13:12:18Z olivier $
- 
+/*      $Id: events.h 24099 2006-12-13 22:01:17Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+
         oroborus - (c) 2001 Ken Lynch
         xfwm4    - (c) 2002-2006 Olivier Fourdan
- 
+
  */
 
 #ifndef INC_EVENTS_H
@@ -26,8 +26,7 @@
 #  include "config.h"
 #endif
 
-#include <sys/types.h>
-#include <sys/time.h>
+#include <X11/X.h>
 #include <X11/Xlib.h>
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
@@ -36,8 +35,9 @@
 #include "screen.h"
 #include "event_filter.h"
 
-void initMenuEventWin (void);
-eventFilterStatus xfwm4_event_filter (XEvent * xevent, gpointer data);
-void initGtkCallbacks (ScreenInfo *);
+void                     initMenuEventWin                       (void);
+eventFilterStatus        xfwm4_event_filter                     (XEvent *, 
+                                                                 gpointer);
+void                     initGtkCallbacks                       (ScreenInfo *);
 
 #endif /* INC_EVENTS_H */
