@@ -1,32 +1,34 @@
-/*      $Id: hints.c 22954 2006-08-30 19:41:26Z olivier $
- 
+/*      $Id: hints.c 25831 2007-06-21 22:28:04Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+
         oroborus - (c) 2001 Ken Lynch
         Metacity - (c) 2001 Havoc Pennington
         xfwm4    - (c) 2002-2006 Olivier Fourdan
- 
+
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <X11/X.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xmd.h>
+
 #include <glib.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
@@ -35,8 +37,10 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <libxfce4util/libxfce4util.h> 
+#include <libxfce4util/libxfce4util.h>
+
 #include "display.h"
+#include "screen.h"
 #include "hints.h"
 
 static gboolean
@@ -73,7 +77,7 @@ static gchar*
 create_name_with_host (DisplayInfo *display_info, const gchar *name, const gchar *hostname)
 {
     gchar *title;
-    
+
     if (strlen (hostname) && (display_info->hostname) && (g_strcasecmp (display_info->hostname, hostname)))
     {
         /* TRANSLATORS: "(on %s)" is like "running on" the name of the other host */
@@ -81,7 +85,7 @@ create_name_with_host (DisplayInfo *display_info, const gchar *name, const gchar
     }
     else
     {
-        title = g_strdup (name);        
+        title = g_strdup (name);
     }
 
     return title;
@@ -100,7 +104,7 @@ getWMState (DisplayInfo *display_info, Window w)
 
     data = NULL;
     state = WithdrawnState;
-    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[WM_STATE], 
+    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[WM_STATE],
                              0, 3L, FALSE, display_info->atoms[WM_STATE],
                              &real_type, &real_format, &items_read, &items_left,
                              (unsigned char **) &data) == Success) && (items_read))
@@ -124,7 +128,7 @@ setWMState (DisplayInfo *display_info, Window w, unsigned long state)
     data[0] = state;
     data[1] = None;
 
-    XChangeProperty (display_info->dpy, w, display_info->atoms[WM_STATE], 
+    XChangeProperty (display_info->dpy, w, display_info->atoms[WM_STATE],
                      display_info->atoms[WM_STATE], 32, PropModeReplace,
                      (unsigned char *) data, 2);
 }
@@ -142,16 +146,16 @@ getMotifHints (DisplayInfo *display_info, Window w)
 
     data = NULL;
     result = NULL;
-    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[MOTIF_WM_HINTS], 0L, MWM_HINTS_ELEMENTS, 
+    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[MOTIF_WM_HINTS], 0L, MWM_HINTS_ELEMENTS,
                 FALSE, display_info->atoms[MOTIF_WM_HINTS], &real_type, &real_format, &items_read,
                 &items_left, (unsigned char **) &data) == Success))
     {
         if (items_read >= MWM_HINTS_ELEMENTS)
-        {    
+        {
             result = g_new0(PropMwmHints, 1);
             memcpy (result, data, sizeof (PropMwmHints));
         }
-        if (data)        
+        if (data)
         {
             XFree (data);
         }
@@ -238,19 +242,19 @@ getHint (DisplayInfo *display_info, Window w, int atom_id, long *value)
     int real_format;
     gboolean success;
 
-    g_return_val_if_fail (((atom_id >= 0) && (atom_id < NB_ATOMS)), FALSE);
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < ATOM_COUNT)), FALSE);
     TRACE ("entering getHint");
 
     success = FALSE;
     *value = 0;
     data = NULL;
 
-    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id], 0L, 1L, 
+    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id], 0L, 1L,
                              FALSE, XA_CARDINAL, &real_type, &real_format, &items_read, &items_left,
                              (unsigned char **) &data) == Success) && (items_read))
     {
         *value = *((long *) data);
-        if (data)        
+        if (data)
         {
             XFree (data);
         }
@@ -262,10 +266,10 @@ getHint (DisplayInfo *display_info, Window w, int atom_id, long *value)
 void
 setHint (DisplayInfo *display_info, Window w, int atom_id, long value)
 {
-    g_return_if_fail ((atom_id >= 0) && (atom_id < NB_ATOMS));
+    g_return_if_fail ((atom_id >= 0) && (atom_id < ATOM_COUNT));
     TRACE ("entering setHint");
 
-    XChangeProperty (display_info->dpy, w, display_info->atoms[atom_id], XA_CARDINAL, 
+    XChangeProperty (display_info->dpy, w, display_info->atoms[atom_id], XA_CARDINAL,
                      32, PropModeReplace, (unsigned char *) &value, 1);
 }
 
@@ -299,16 +303,16 @@ getDesktopLayout (DisplayInfo *display_info, Window root, int ws_count, NetWmDes
             rows = (unsigned long) *ptr++;
             start = (items_read >= 4) ? (unsigned long) *ptr++ : NET_WM_TOPLEFT;
 
-            if (orientation > NET_WM_ORIENTATION_VERT) 
+            if (orientation > NET_WM_ORIENTATION_VERT)
             {
                 break;
             }
-            
+
             if (start > NET_WM_BOTTOMLEFT)
             {
                 break;
             }
-            
+
             if ((rows == 0) && (cols == 0))
             {
                 break;
@@ -426,6 +430,7 @@ setNetSupportedHint (DisplayInfo *display_info, Window root, Window check_win)
     atoms[i++] = display_info->atoms[NET_WM_ICON];
     atoms[i++] = display_info->atoms[NET_WM_ICON_GEOMETRY];
     atoms[i++] = display_info->atoms[NET_WM_ICON_NAME];
+    atoms[i++] = display_info->atoms[NET_WM_MOVERESIZE];
     atoms[i++] = display_info->atoms[NET_WM_NAME];
     atoms[i++] = display_info->atoms[NET_WM_STATE];
     atoms[i++] = display_info->atoms[NET_WM_STATE_ABOVE];
@@ -442,6 +447,8 @@ setNetSupportedHint (DisplayInfo *display_info, Window root, Window check_win)
     atoms[i++] = display_info->atoms[NET_WM_STATE_STICKY];
     atoms[i++] = display_info->atoms[NET_WM_STRUT];
     atoms[i++] = display_info->atoms[NET_WM_STRUT_PARTIAL];
+    atoms[i++] = display_info->atoms[NET_WM_SYNC_REQUEST];
+    atoms[i++] = display_info->atoms[NET_WM_SYNC_REQUEST_COUNTER];
     atoms[i++] = display_info->atoms[NET_WM_USER_TIME];
     atoms[i++] = display_info->atoms[NET_WM_WINDOW_TYPE];
     atoms[i++] = display_info->atoms[NET_WM_WINDOW_TYPE_DESKTOP];
@@ -456,11 +463,11 @@ setNetSupportedHint (DisplayInfo *display_info, Window root, Window check_win)
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
     atoms[i++] = display_info->atoms[NET_STARTUP_ID];
 #endif
-
+    g_assert (i < 64);
     data[0] = check_win;
-    XChangeProperty (display_info->dpy, root, display_info->atoms[NET_SUPPORTED], 
+    XChangeProperty (display_info->dpy, root, display_info->atoms[NET_SUPPORTED],
                      XA_ATOM, 32, PropModeReplace, (unsigned char *) atoms, i);
-    XChangeProperty (display_info->dpy, check_win, display_info->atoms[NET_SUPPORTING_WM_CHECK], 
+    XChangeProperty (display_info->dpy, check_win, display_info->atoms[NET_SUPPORTING_WM_CHECK],
                      XA_WINDOW, 32, PropModeReplace, (unsigned char *) data, 1);
     XChangeProperty (display_info->dpy, root, display_info->atoms[NET_SUPPORTING_WM_CHECK],
                      XA_WINDOW, 32, PropModeReplace, (unsigned char *) data, 1);
@@ -479,11 +486,11 @@ getAtomList (DisplayInfo *display_info, Window w, int atom_id, Atom ** atoms_p,
     *atoms_p = NULL;
     *n_atoms_p = 0;
 
-    g_return_val_if_fail (((atom_id >= 0) && (atom_id < NB_ATOMS)), FALSE);
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < ATOM_COUNT)), FALSE);
     TRACE ("entering getAtomList()");
 
-    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id], 
-                             0, G_MAXLONG, FALSE, XA_ATOM, &type, &format, &n_atoms, 
+    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id],
+                             0, G_MAXLONG, FALSE, XA_ATOM, &type, &format, &n_atoms,
                              &bytes_after, (unsigned char **) &data) != Success) || (type == None))
     {
         return FALSE;
@@ -519,10 +526,10 @@ getCardinalList (DisplayInfo *display_info, Window w, int atom_id, unsigned long
     *cardinals_p = NULL;
     *n_cardinals_p = 0;
 
-    g_return_val_if_fail (((atom_id >= 0) && (atom_id < NB_ATOMS)), FALSE);
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < ATOM_COUNT)), FALSE);
     TRACE ("entering getCardinalList()");
 
-    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id], 
+    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id],
                              0, G_MAXLONG, FALSE, XA_CARDINAL,
                              &type, &format, &n_cardinals, &bytes_after,
                              (unsigned char **) &data) != Success) || (type == None))
@@ -554,12 +561,12 @@ setNetWorkarea (DisplayInfo *display_info, Window root, int nb_workspaces, int w
     ptr = data;
     for (i = 0; i < j; i++)
     {
-        *ptr++ = (unsigned long) m[LEFT];
-        *ptr++ = (unsigned long) m[TOP];
-        *ptr++ = (unsigned long) (width  - (m[LEFT] + m[RIGHT]));
-        *ptr++ = (unsigned long) (height - (m[TOP] + m[BOTTOM]));
+        *ptr++ = (unsigned long) m[STRUTS_LEFT];
+        *ptr++ = (unsigned long) m[STRUTS_TOP];
+        *ptr++ = (unsigned long) (width  - (m[STRUTS_LEFT] + m[STRUTS_RIGHT]));
+        *ptr++ = (unsigned long) (height - (m[STRUTS_TOP]  + m[STRUTS_BOTTOM]));
     }
-    XChangeProperty (display_info->dpy, root, display_info->atoms[NET_WORKAREA], 
+    XChangeProperty (display_info->dpy, root, display_info->atoms[NET_WORKAREA],
                      XA_CARDINAL, 32, PropModeReplace, (unsigned char *) data, j * 4);
     g_free (data);
 }
@@ -574,7 +581,7 @@ setNetFrameExtents (DisplayInfo *display_info, Window w, int top, int left, int
     data[1] = (unsigned long) right;
     data[2] = (unsigned long) top;
     data[3] = (unsigned long) bottom;
-    XChangeProperty (display_info->dpy, w, display_info->atoms[NET_FRAME_EXTENTS], 
+    XChangeProperty (display_info->dpy, w, display_info->atoms[NET_FRAME_EXTENTS],
                      XA_CARDINAL, 32, PropModeReplace, (unsigned char *) data, 4);
 }
 
@@ -599,10 +606,10 @@ initNetDesktopInfo (DisplayInfo *display_info, Window root, int workspace, int w
 void
 setUTF8StringHint (DisplayInfo *display_info, Window w, int atom_id, const gchar *val)
 {
-    g_return_if_fail ((atom_id >= 0) && (atom_id < NB_ATOMS));
+    g_return_if_fail ((atom_id >= 0) && (atom_id < ATOM_COUNT));
     TRACE ("entering setUTF8StringHint");
 
-    XChangeProperty (display_info->dpy, w, display_info->atoms[atom_id], 
+    XChangeProperty (display_info->dpy, w, display_info->atoms[atom_id],
                      display_info->atoms[UTF8_STRING], 8, PropModeReplace,
                      (unsigned char *) val, strlen (val) + 1);
 }
@@ -643,7 +650,7 @@ text_property_to_utf8 (DisplayInfo *display_info, const XTextProperty * prop)
     TRACE ("entering text_property_to_utf8");
 
     list = NULL;
-    count = gdk_text_property_to_utf8_list (gdk_x11_xatom_to_atom (prop->encoding), 
+    count = gdk_text_property_to_utf8_list (gdk_x11_xatom_to_atom (prop->encoding),
                                             prop->format, prop->value, prop->nitems, &list);
     if (count == 0)
     {
@@ -695,12 +702,12 @@ getUTF8StringData (DisplayInfo *display_info, Window w, int atom_id, gchar **str
     unsigned char *str;
     unsigned long n_items;
 
-    g_return_val_if_fail (((atom_id >= 0) && (atom_id < NB_ATOMS)), FALSE);
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < ATOM_COUNT)), FALSE);
     TRACE ("entering getUTF8StringData");
 
     *str_p = NULL;
-    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id], 
-                             0, G_MAXLONG, FALSE, display_info->atoms[UTF8_STRING], &type, 
+    if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id],
+                             0, G_MAXLONG, FALSE, display_info->atoms[UTF8_STRING], &type,
                              &format, &n_items, &bytes_after, (unsigned char **) &str) != Success) || (type == None))
     {
         TRACE ("no UTF8_STRING property found");
@@ -727,8 +734,8 @@ gboolean
 getUTF8String (DisplayInfo *display_info, Window w, int atom_id, gchar **str_p, int *length)
 {
     char *xstr;
-    
-    g_return_val_if_fail (((atom_id >= 0) && (atom_id < NB_ATOMS)), FALSE);
+
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < ATOM_COUNT)), FALSE);
     TRACE ("entering getUTF8String");
 
     if (!getUTF8StringData (display_info, w, atom_id, &xstr, length))
@@ -742,7 +749,7 @@ getUTF8String (DisplayInfo *display_info, Window w, int atom_id, gchar **str_p,
     /* gmalloc the returned string */
     *str_p = internal_utf8_strndup (xstr, MAX_STR_LENGTH);
     XFree (xstr);
-    
+
     if (!g_utf8_validate (*str_p, -1, NULL))
     {
         TRACE ("getUTF8String() returned invalid UTF-8 characters");
@@ -757,7 +764,7 @@ getUTF8String (DisplayInfo *display_info, Window w, int atom_id, gchar **str_p,
     {
         xfce_utf8_remove_controls((gchar *) *str_p, -1, NULL);
     }
-    
+
     return TRUE;
 }
 
@@ -769,7 +776,7 @@ getUTF8StringList (DisplayInfo *display_info, Window w, int atom_id, gchar ***st
     guint i;
     int length;
 
-    g_return_val_if_fail (((atom_id >= 0) && (atom_id < NB_ATOMS)), FALSE);
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < ATOM_COUNT)), FALSE);
 
     TRACE ("entering getUTF8StringList");
 
@@ -780,7 +787,7 @@ getUTF8StringList (DisplayInfo *display_info, Window w, int atom_id, gchar ***st
     {
         return FALSE;
     }
-    
+
     i = 0;
     while (i < length)
     {
@@ -794,10 +801,10 @@ getUTF8StringList (DisplayInfo *display_info, Window w, int atom_id, gchar ***st
     {
         *n_items = *n_items + 1;
     }
- 
+
     retval = g_new0 (gchar *, *n_items + 1);
     ptr = xstr;
-    
+
     for (i = 0; i < *n_items; i++)
     {
         if (g_utf8_validate (ptr, -1, NULL))
@@ -812,7 +819,7 @@ getUTF8StringList (DisplayInfo *display_info, Window w, int atom_id, gchar ***st
         ptr += strlen (ptr) + 1;
     }
     XFree (xstr);
-    *str_p = retval;    
+    *str_p = retval;
 
     return TRUE;
 }
@@ -852,7 +859,7 @@ getWindowName (DisplayInfo *display_info, Window w, gchar **title)
     gchar *machine;
     gchar *name;
     gboolean status;
-    
+
     TRACE ("entering getWindowName");
 
     g_return_val_if_fail (title != NULL, FALSE);
@@ -921,7 +928,7 @@ getClientLeader (DisplayInfo *display_info, Window window)
     g_return_val_if_fail (window != None, None);
 
     client_leader = None;
-    if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[WM_CLIENT_LEADER], 
+    if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[WM_CLIENT_LEADER],
                             0L, 1L, FALSE, AnyPropertyType, &actual_type, &actual_format, &nitems,
                             &bytes_after, (unsigned char **) &prop) == Success)
     {
@@ -949,9 +956,9 @@ getNetWMUserTime (DisplayInfo *display_info, Window window, Time *time)
 
     TRACE ("entering getNetWMUserTime");
 
-    g_return_val_if_fail (window != None, None);
+    g_return_val_if_fail (window != None, FALSE);
 
-    if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[NET_WM_USER_TIME], 
+    if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[NET_WM_USER_TIME],
                             0L, 1L, FALSE, XA_CARDINAL, &actual_type, &actual_format, &nitems,
                             &bytes_after, (unsigned char **) &data) == Success)
     {
@@ -963,7 +970,8 @@ getNetWMUserTime (DisplayInfo *display_info, Window window, Time *time)
             return TRUE;
         }
     }
-    *time = 0L;
+    *time = (Time) 0;
+
     return FALSE;
 }
 
@@ -1031,8 +1039,8 @@ getKDEIcon (DisplayInfo *display_info, Window window, Pixmap * pixmap, Pixmap *
     *mask = None;
 
     icons = NULL;
-    if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[KWM_WIN_ICON], 
-                            0L, G_MAXLONG, FALSE, display_info->atoms[KWM_WIN_ICON], &type, 
+    if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[KWM_WIN_ICON],
+                            0L, G_MAXLONG, FALSE, display_info->atoms[KWM_WIN_ICON], &type,
                             &format, &nitems, &bytes_after, (unsigned char **)&data) != Success)
     {
         return FALSE;
@@ -1062,14 +1070,14 @@ getRGBIconData (DisplayInfo *display_info, Window window, unsigned long **data,
     int format;
     unsigned long bytes_after;
 
-    if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[NET_WM_ICON], 
+    if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[NET_WM_ICON],
                             0L, G_MAXLONG, FALSE, XA_CARDINAL, &type, &format, nitems,
                             &bytes_after, (unsigned char **) data) != Success)
     {
         *data = NULL;
         return FALSE;
     }
-    
+
     if (type != XA_CARDINAL)
     {
         if (*data)
@@ -1087,10 +1095,10 @@ gboolean
 getOpacity (DisplayInfo *display_info, Window window, guint *opacity)
 {
     long val;
-    
+
     g_return_val_if_fail (window != None, FALSE);
     g_return_val_if_fail (opacity != NULL, FALSE);
-    TRACE ("entering getOpacity");    
+    TRACE ("entering getOpacity");
 
     val = 0;
     if (getHint (display_info, window, NET_WM_WINDOW_OPACITY, &val))
@@ -1136,11 +1144,11 @@ setXAtomManagerOwner (DisplayInfo *display_info, Atom atom, Window root, Window
         ev.type = ClientMessage;
         ev.message_type = atom;
         ev.format = 32;
-        ev.data.l[0] = server_time;
-        ev.data.l[1] = atom;
-        ev.data.l[2] = w;
-        ev.data.l[3] = 0;
-        ev.data.l[4] = 0;
+        ev.data.l[0] = (long) server_time;
+        ev.data.l[1] = (long) atom;
+        ev.data.l[2] = (long) w;
+        ev.data.l[3] = (long) 0L;
+        ev.data.l[4] = (long) 0L;
         ev.window = root;
 
         XSendEvent (display_info->dpy, root, FALSE, StructureNotifyMask, (XEvent *) &ev);
@@ -1154,34 +1162,81 @@ setXAtomManagerOwner (DisplayInfo *display_info, Atom atom, Window root, Window
 gboolean
 setAtomIdManagerOwner (DisplayInfo *display_info, int atom_id, Window root, Window w)
 {
-    g_return_val_if_fail (((atom_id >= 0) && (atom_id < NB_ATOMS)), FALSE);
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < ATOM_COUNT)), FALSE);
 
     return setXAtomManagerOwner(display_info, display_info->atoms[atom_id], root, w);
 }
 
+void
+updateXserverTime (DisplayInfo *display_info)
+{
+    ScreenInfo *screen_info;
+    char c = '\0';
+
+    g_return_if_fail (display_info);
+
+    XChangeProperty (display_info->dpy, display_info->timestamp_win,
+                     display_info->atoms[XFWM4_TIMESTAMP_PROP],
+                     display_info->atoms[XFWM4_TIMESTAMP_PROP],
+                     8, PropModeReplace, (unsigned char *) &c, 1);
+}
+
+Time
+getXServerTime (DisplayInfo *display_info)
+{
+    ScreenInfo *screen_info;
+    XEvent xevent;
+    Time timestamp;
+
+    g_return_val_if_fail (display_info, (Time) CurrentTime);
+    timestamp = myDisplayGetCurrentTime (display_info);
+    if (timestamp == (Time) CurrentTime)
+    {
+        screen_info = myDisplayGetDefaultScreen (display_info);
+        g_return_val_if_fail (screen_info,  (Time) CurrentTime);
+
+        TRACE ("getXServerTime: Using X server roundtrip");
+        updateXserverTime (display_info);
+        XWindowEvent (display_info->dpy, display_info->timestamp_win, PropertyChangeMask, &xevent);
+        timestamp = (Time) myDisplayUpdateCurrentTime (display_info, &xevent);
+    }
+
+    TRACE ("getXServerTime gives timestamp=%u", (unsigned int) timestamp);
+    return timestamp;
+}
+
 #ifdef ENABLE_KDE_SYSTRAY_PROXY
 gboolean
-checkKdeSystrayWindow(DisplayInfo *display_info, Window window)
+checkKdeSystrayWindow (DisplayInfo *display_info, Window window)
 {
     Atom actual_type;
     int actual_format;
     unsigned long nitems;
     unsigned long bytes_after;
+    unsigned char *data;
     Window trayIconForWindow;
 
-    TRACE ("entering GetWindowRole");
+    TRACE ("entering checkKdeSystrayWindow");
     g_return_val_if_fail (window != None, FALSE);
-    
-    XGetWindowProperty(display_info->dpy, window, display_info->atoms[KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR], 
-                       0L, sizeof(Window), FALSE, XA_WINDOW, &actual_type, &actual_format, 
-                       &nitems, &bytes_after, (unsigned char **) &trayIconForWindow);
 
-    if ((actual_format == None) || 
-        (actual_type != XA_WINDOW) || 
+    XGetWindowProperty(display_info->dpy, window, display_info->atoms[KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR],
+                       0L, sizeof(Window), FALSE, XA_WINDOW, &actual_type, &actual_format,
+                       &nitems, &bytes_after, (unsigned char **) &data);
+
+    trayIconForWindow = None;
+    if (data)
+    {
+        trayIconForWindow = *((Window *) data);
+        XFree (data);
+    }
+
+    if ((actual_format == None) ||
+        (actual_type != XA_WINDOW) ||
         (trayIconForWindow == None))
     {
         return FALSE;
     }
+
     return TRUE;
 }
 
@@ -1198,11 +1253,11 @@ sendSystrayReqDock(DisplayInfo *display_info, Window window, Window systray)
     xev.window = systray;
     xev.message_type = display_info->atoms[NET_SYSTEM_TRAY_OPCODE];
     xev.format = 32;
-    xev.data.l[0] = CurrentTime;
-    xev.data.l[1] = 0; /* SYSTEM_TRAY_REQUEST_DOCK */
-    xev.data.l[2] = window;
-    xev.data.l[3] = 0; /* Nada */
-    xev.data.l[4] = 0; /* Niet */
+    xev.data.l[0] = (long) myDisplayGetCurrentTime (display_info);
+    xev.data.l[1] = (long) 0L; /* SYSTEM_TRAY_REQUEST_DOCK */
+    xev.data.l[2] = (long) window;
+    xev.data.l[3] = (long) 0L; /* Nada */
+    xev.data.l[4] = (long) 0L; /* Niet */
 
     XSendEvent (display_info->dpy, systray, FALSE, NoEventMask, (XEvent *) & xev);
 }
@@ -1255,3 +1310,44 @@ getWindowStartupId (DisplayInfo *display_info, Window w, gchar **startup_id)
     return FALSE;
 }
 #endif
+
+#ifdef HAVE_XSYNC
+gboolean
+getXSyncCounter (DisplayInfo *display_info, Window window, XSyncCounter *counter)
+{
+    long val;
+
+    g_return_val_if_fail (window != None, FALSE);
+    g_return_val_if_fail (counter != NULL, FALSE);
+    TRACE ("entering getXSyncCounter");
+
+    val = 0;
+    if (getHint (display_info, window, NET_WM_SYNC_REQUEST_COUNTER, &val))
+    {
+        *counter = (XSyncCounter) val;
+        return TRUE;
+    }
+
+    return FALSE;
+}
+
+void
+sendXSyncRequest (DisplayInfo *display_info, Window window, XSyncValue value)
+{
+    XClientMessageEvent xev;
+
+    g_return_if_fail (window != None);
+    TRACE ("entering getXSyncCounter");
+
+    xev.type = ClientMessage;
+    xev.window = window;
+    xev.message_type = display_info->atoms[WM_PROTOCOLS];
+    xev.format = 32;
+    xev.data.l[0] = (long) display_info->atoms[NET_WM_SYNC_REQUEST];
+    xev.data.l[1] = (long) myDisplayGetCurrentTime (display_info);
+    xev.data.l[2] = (long) XSyncValueLow32 (value);
+    xev.data.l[3] = (long) XSyncValueHigh32 (value);
+    xev.data.l[4] = (long) 0L;
+    XSendEvent (display_info->dpy, window, FALSE, NoEventMask, (XEvent *) &xev);
+}
+#endif /* HAVE_XSYNC */
