@@ -1,4 +1,4 @@
-/*      $Id: keyboard.c 23826 2006-11-11 15:20:18Z olivier $
+/*      $Id: keyboard.c 25673 2007-05-02 21:08:49Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -261,9 +261,9 @@ initModifiers (Display * dpy)
     XModifierKeymap *modmap;
     KeySym *keymap;
     unsigned int keycode;
-    int keysyms_per_keycode;
     int min_keycode;
     int max_keycode;
+    int keysyms_per_keycode;
     int i;
 
     AltMask = 0;
@@ -280,41 +280,61 @@ initModifiers (Display * dpy)
     modmap = XGetModifierMapping (dpy);
     keymap = XGetKeyboardMapping (dpy, min_keycode, max_keycode - min_keycode + 1, &keysyms_per_keycode);
 
-    for (i = 3 * modmap->max_keypermod; i < 8 * modmap->max_keypermod; i++)
-    {
-        keycode = modmap->modifiermap[i];
-        if ((keycode >= min_keycode) && (keycode <= max_keycode))
+    if (modmap && keymap)
+    {    
+        for (i = 3 * modmap->max_keypermod; i < 8 * modmap->max_keypermod; i++)
         {
-            int j;
-            KeySym *syms = keymap + (keycode - min_keycode) * keysyms_per_keycode;
-
-            for (j = 0; j < keysyms_per_keycode; j++)
+            keycode = modmap->modifiermap[i];
+            if ((keycode >= min_keycode) && (keycode <= max_keycode))
             {
-                if (syms[j] == XK_Num_Lock)
+                int j;
+                KeySym *syms = keymap + (keycode - min_keycode) * keysyms_per_keycode;
+
+                for (j = 0; j < keysyms_per_keycode; j++)
                 {
-                    NumLockMask |= (1 << ( i / modmap->max_keypermod));
-                }
-                else if (syms[j] == XK_Scroll_Lock)
-                {
-                    ScrollLockMask |= (1 << ( i / modmap->max_keypermod));
-                }
-                else if ((syms[j] == XK_Super_L) || (syms[j] == XK_Super_R))
-                {
-                    SuperMask |= (1 << ( i / modmap->max_keypermod));
-                }
-                else if ((syms[j] == XK_Hyper_L) || (syms[j] == XK_Hyper_R))
-                {
-                    HyperMask |= (1 << ( i / modmap->max_keypermod));
-                }       
-                else if ((syms[j] == XK_Meta_L) || (syms[j] == XK_Meta_R))
-                {
-                    MetaMask |= (1 << ( i / modmap->max_keypermod));
-                }
-                else if ((syms[j] == XK_Alt_L) || (syms[j] == XK_Alt_R))
-                {
-                    AltMask |= (1 << ( i / modmap->max_keypermod));
+                    if (!NumLockMask && (syms[j] == XK_Num_Lock))
+                    {
+                        NumLockMask = (1 << (i / modmap->max_keypermod));
+                    }
+                    else if (!ScrollLockMask && (syms[j] == XK_Scroll_Lock))
+                    {
+                        ScrollLockMask = (1 << (i / modmap->max_keypermod));
+                    }
+                    else if (!AltMask && ((syms[j] == XK_Alt_L) || (syms[j] == XK_Alt_R)))
+                    {
+                        AltMask = (1 << (i / modmap->max_keypermod));
+                    }
+                    else if (!SuperMask && ((syms[j] == XK_Super_L) || (syms[j] == XK_Super_R)))
+                    {
+                        SuperMask = (1 << (i / modmap->max_keypermod));
+                    }
+                    else if (!HyperMask && ((syms[j] == XK_Hyper_L) || (syms[j] == XK_Hyper_R)))
+                    {
+                        HyperMask = (1 << (i / modmap->max_keypermod));
+                    }       
+                    else if (!MetaMask && ((syms[j] == XK_Meta_L) || (syms[j] == XK_Meta_R)))
+                    {
+                        MetaMask = (1 << (i / modmap->max_keypermod));
+                    }
                 }
             }
         }
     }
+
+    /* Cleanup memory */
+    if (modmap)
+    {
+        XFreeModifiermap (modmap);
+    }
+
+    if (keymap)
+    {
+        XFree (keymap);
+    }
+
+    /* In case we didn't find AltMask, use Mod1Mask */
+    if (AltMask == 0)
+    {
+        AltMask = Mod1Mask;
+    }
 }
