@@ -1,22 +1,22 @@
-/*      $Id: menu.h 20527 2006-03-24 12:26:41Z olivier $
- 
+/*      $Id: menu.h 23704 2006-11-04 22:36:42Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+
         Metacity - (c) 2001 Havoc Pennington
         xfwm4    - (c) 2002-2006 Olivier Fourdan
- 
+
  */
 
 #ifndef INC_MENU_H
@@ -29,7 +29,8 @@
 #include <X11/Xlib.h>
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
-#include <libxfcegui4/libxfcegui4.h>
+
+#include "event_filter.h"
 
 typedef enum
 {
@@ -80,7 +81,7 @@ struct _Menu
 {
     GdkScreen *screen;
     GtkWidget *menu;
-    XfceFilterSetup *filter_setup;
+    eventFilterSetup *filter_setup;
     MenuFunc func;
     MenuOp ops;
     MenuOp insensitive;
@@ -88,27 +89,27 @@ struct _Menu
     gpointer data;
 };
 
-Menu *menu_default             (GdkScreen *,
-                                Window,
-                                MenuOp, 
-                                MenuOp, 
-                                MenuFunc, 
-                                gint, 
-                                gint, 
-                                gchar **, 
-                                gint, 
-                                XfceFilterSetup*, 
-                                gpointer);
-Menu *menu_connect             (Menu * menu);
-GtkWidget *menu_item_connect   (GtkWidget *, 
-                                MenuData *);
-gboolean menu_is_opened        (void);
-gboolean menu_check_and_close  (void);
-gboolean menu_popup            (Menu *, 
-                                int, 
-                                int, 
-                                int, 
-                                guint32);
-void menu_free                 (Menu *);
+Menu                    *menu_default                           (GdkScreen *,
+                                                                 Window,
+                                                                 MenuOp,
+                                                                 MenuOp,
+                                                                 MenuFunc,
+                                                                 gint,
+                                                                 gint,
+                                                                 gchar **,
+                                                                 gint,
+                                                                 eventFilterSetup*,
+                                                                 gpointer);
+Menu                    *menu_connect                           (Menu *);
+GtkWidget               *menu_item_connect                      (GtkWidget *,
+                                                                 MenuData *);
+gboolean                 menu_is_opened                         (void);
+gboolean                 menu_check_and_close                   (void);
+gboolean                 menu_popup                             (Menu *,
+                                                                 int,
+                                                                 int,
+                                                                 int,
+                                                                 guint32);
+void                     menu_free                              (Menu *);
 
 #endif /* INC_MENU_H */
