@@ -1,4 +1,4 @@
-/*      $Id: misc.h 23704 2006-11-04 22:36:42Z olivier $
+/*      $Id: misc.h 24099 2006-12-13 22:01:17Z olivier $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -23,10 +23,7 @@
 #  include "config.h"
 #endif
 
-#include <sys/types.h>
-#include <sys/time.h>
-#include <time.h>
-
+#include <X11/X.h>
 #include <X11/Xlib.h>
 #include <glib.h>
 #include "screen.h"
