@@ -1,19 +1,19 @@
-/*      $Id: session.h 21727 2006-05-20 11:33:06Z olivier $
- 
+/*      $Id: session.h 23704 2006-11-04 22:36:42Z olivier $
+
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
         the Free Software Foundation; either version 2, or (at your option)
         any later version.
- 
+
         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
         GNU General Public License for more details.
- 
+
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- 
+
         xfwm4    - (c) 2002-2006 Olivier Fourdan
  */
 
@@ -29,36 +29,31 @@
 /*
  *  Save window states to file which name is given in argument.
  */
-gboolean 
-sessionSaveWindowStates                (DisplayInfo *, 
-                                        gchar *);
+gboolean                sessionSaveWindowStates                 (DisplayInfo *,
+                                                                 gchar *);
 
 /*
  *  Load window states to file which name is given in argument.
  */
-gboolean
-sessionLoadWindowStates                (gchar *);
+gboolean                sessionLoadWindowStates                 (gchar *);
 
 /*
  * Free allocated structure. Should be called before xfwm4 dies
  */
-void
-sessionFreeWindowStates                (void);
+void                    sessionFreeWindowStates                 (void);
 
 /*
  * Search for existing client in saved session and update
  * relevant client fields if found.
  */
-gboolean
-sessionMatchWinToSM                    (Client *);
+gboolean                sessionMatchWinToSM                     (Client *);
 
 /*
  * Initiate session, connect to session manager and
  * load saved states if the connection succeeds.
  */
-int
-sessionStart                           (int, 
-                                        char **,
-                                        DisplayInfo *);
+int                     sessionStart                            (int,
+                                                                 char **,
+                                                                 DisplayInfo *);
 
 #endif /* INC_CLIENT_H */
